SCRIPT_REAL("

import datetime
from scipy import optimize

def eir_derivative_func(rate, pmts, dates):
#     Find the derivative or the EIR function, used for calculating
#     Newton's method:

#     http://en.wikipedia.org/wiki/Newton's_method

#     EIR = cf*(1+rate)^d
#     f'rate = cf*d*(rate+1)^(d-1)

#     Credit: http://mail.scipy.org/pipermail/numpy-discussion/2009-May/042736.html
#     Also see: http://mftransparency.org/wp-content/uploads/2010/10/xirr_example.py_.txt

    dcf=[]
    count = 0
    for i, cf in enumerate(pmts):
        d = dates[i] - dates[0]
        n = (-d.days / 365.)
        dcf.append( cf * n * (rate + 1)**(n - 1) )
    return sum(dcf)

def eir_func(rate, pmts, dates):
#     Loop through the dates and calculate a discounted cashflow total

#     This is very similar to eir_derivative_func, but with the EIR formula,
#     rather than f'rate.

#     Credit: http://mail.scipy.org/pipermail/numpy-discussion/2009-May/042736.html
#     Also see: http://mftransparency.org/wp-content/uploads/2010/10/xirr_example.py_.txt

    dcf=[]
    for i, cf in enumerate(pmts):
        d = dates[i] - dates[0]
        n = (-d.days / 365.)
        discounted_period = cf * (rate + 1)**(n)
        dcf.append( discounted_period )

    discounted_cashflow = sum(dcf)
    return discounted_cashflow

def xirr(payments,dates,guess=0.1):
    # list must start with deposit
    if payments[0] > 0:
        return 0
    # invert guess for lossing assets
    if sum(payments) < 0:
        guess = -0.1
    f = lambda x: eir_func(x, payments, dates)
    derivative = lambda x: eir_derivative_func(x, payments, dates)
    return optimize.newton(f, guess, fprime=derivative, args=(),
            tol=0.00000000001, maxiter=50)

def multi_format_date_parse(text):
    if text.find('000') > 0:
        dateSplit = text.split('-');
        text = dateSplit[0] + '-' + dateSplit[1][2:] +  '-' + dateSplit[2]
    for fmt in ('%Y-%m-%d', '%Y-%m-%dT%H:%M:%SZ'):
        try:
            return datetime.datetime.strptime(text, fmt).date()
        except ValueError:
            pass
    raise ValueError('no valid date format found')


# Define Arguments (Tableau)
cashflows      = _arg1
date_list      = _arg2
# Debugging
#print(cashflows)
#print(date_list)
#print(date_list)
#print(date_parameter)
transaction_dates = []

nav = date_list[-1]
if nav > 0 and cashflows[0] <= 0:
    
    # Format dates
    for idx, i in enumerate(date_list):
        transaction_dates.append(multi_format_date_parse(date_list[idx]))
    
    # Calculate XIRR
    if sum(cashflows) < 0:
        return xirr(cashflows, transaction_dates, guess=-0.1)
    else:
        return xirr(cashflows, transaction_dates)
else:
    return 0
",
// Assign Tableau Columns
    // Values column
    SUM([Cashflow]),
    // Date column
    ATTR([DATE STRING])
)